(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

(* Util for Big_int *)
open Big_int

(* Some constants *)

let bi i = big_int_of_int i
let bi_0 = bi 0
let bi_1 = bi 1
(*
let bi_4 = bi 4
*)
let bi_size_of_word = bi Machine.size_of_word

(* infix operators *)

let (=^) bi1 bi2 = eq_big_int bi1 bi2
let (<>^) bi1 bi2 = not (bi1 =^ bi2)
let (<^) bi1 bi2 = lt_big_int bi1 bi2
let (>^) bi1 bi2 = gt_big_int bi1 bi2
let (<=^) bi1 bi2 = le_big_int bi1 bi2
let (>=^) bi1 bi2 = ge_big_int bi1 bi2

let (+^) bi1 bi2 = add_big_int bi1 bi2
let (-^) bi1 bi2 = sub_big_int bi1 bi2
let ( *^) bi1 bi2 = mult_big_int bi1 bi2
let (/^) bi1 bi2 = div_big_int bi1 bi2
let (%^) bi1 bi2 = mod_big_int bi1 bi2
let (/%^) bi1 bi2 = quomod_big_int bi1 bi2

let (~^) bi1 = minus_big_int bi1

(* operators *)

let div_floor_big_int bi1 bi2 =
	let q, r = bi1 /%^ bi2 in
	if r =^ bi_0 then
		q
	else

	if (sign_big_int bi1) * (sign_big_int bi2) < 0 then
		q -^ bi_1
	else 
		q

let div_ceil_big_int bi1 bi2 =
	let q, r = bi1 /%^ bi2 in
	if r =^ bi_0 then
		q
	else

	if (sign_big_int bi1) * (sign_big_int bi2) < 0 then
		q
	else 
		q -^ bi_1


(* Converter *)

let big_int_of_nativeint shifter n =
	let n1 = Nativeint.logand n 0x3n in
	let i1 = Nativeint.to_int n1 in

	let n2 = shifter n 2 in
	let i2 = Nativeint.to_int n2 in

	let b1 = big_int_of_int i1 in
	let b2 = big_int_of_int i2 in

	b1 +^ (b2 *^ (bi 4))

let big_int_of_nativeint_signed n =
	big_int_of_nativeint Nativeint.shift_right n

let big_int_of_nativeint_unsigned n =
	big_int_of_nativeint Nativeint.shift_right_logical n

(* Output *)

let output_big_int ppf n =
	Format.fprintf ppf "%s" (string_of_big_int n)

let get_hex_of_big_int r =
	     if r =^ (bi 0) then "0"
	else if r =^ (bi 1) then "1"
	else if r =^ (bi 2) then "2"
	else if r =^ (bi 3) then "3"
	else if r =^ (bi 4) then "4"
	else if r =^ (bi 5) then "5"
	else if r =^ (bi 6) then "6"
	else if r =^ (bi 7) then "7"
	else if r =^ (bi 8) then "8"
	else if r =^ (bi 9) then "9"
	else if r =^ (bi 10) then "a"
	else if r =^ (bi 11) then "b"
	else if r =^ (bi 12) then "c"
	else if r =^ (bi 13) then "d"
	else if r =^ (bi 14) then "e"
	else if r =^ (bi 15) then "f"
	else failwith "x % 16 > 16?"

let rec string_of_big_int_x_aux n =

	let q, r = quomod_big_int n (big_int_of_int 16) in

	let pre =
		if eq_big_int q zero_big_int then
			"0x"
		else
			string_of_big_int_x_aux q
	in

	pre ^ (get_hex_of_big_int r)

let string_of_big_int_x n =
	let s = sign_big_int n in
	let n = abs_big_int n in

	let str = string_of_big_int_x_aux n in

	if s < 0 then
		"-" ^ str
	else
		str

let output_big_int_x ppf n =
	let s = string_of_big_int_x n in

	Format.fprintf ppf "%s" s
