(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Machine
open Instructions

(*************************************
	Disassemble prefixes
 *************************************)

let empty_prefixes =
{
	g1_opt = None;
	g2_opt = None;
	g3_opt = None;
	g4_opt = None;
}

exception Invalid_prefix
exception Not_prefix

let add_prefix_group1 p prefixes =
	if prefixes.g1_opt = None then { prefixes with g1_opt = Some p }
	else raise Invalid_prefix
let add_prefix_group2 p prefixes =
	if prefixes.g2_opt = None then { prefixes with g2_opt = Some p }
	else raise Invalid_prefix
let add_prefix_group3 p prefixes =
	if prefixes.g3_opt = None then { prefixes with g3_opt = Some p }
	else raise Invalid_prefix
let add_prefix_group4 p prefixes =
	if prefixes.g4_opt = None then { prefixes with g4_opt = Some p }
	else raise Invalid_prefix

let add_prefix b prefixes =
	let adder = 
	match b with
(* group 1 *)
	| 0xF0 -> add_prefix_group1 LOCK
	| 0xF2 -> add_prefix_group1 REPNE_REPNZ
	| 0xF3 -> add_prefix_group1 REP_or_REPE_REPZ

(* group 2 *)
	| 0x2E -> add_prefix_group2 WITH_CS_or_BRANCH_NOT_TAKEN
	| 0x36 -> add_prefix_group2 WITH_SS
	| 0x3E -> add_prefix_group2 WITH_DS_or_BRANCH_TAKEN
	| 0x26 -> add_prefix_group2 WITH_ES
	| 0x64 -> add_prefix_group2 WITH_FS
	| 0x65 -> add_prefix_group2 WITH_GS

(* group 3 *)
	| 0x66 -> add_prefix_group3 OPSIZE_OVERRIDE

(* group 4 *)
	| 0x67 -> add_prefix_group4 ADSIZE_OVERRIDE

	| _ -> raise Not_prefix

	in
	adder prefixes

let rec get_prefixes_sub seg off prefixes =
	let b = get_byte seg off in

	try
		let prefixes = add_prefix b prefixes in
		get_prefixes_sub seg (off + 1) prefixes
	with Not_prefix ->
		(off, prefixes)

let get_prefixes seg off =
	get_prefixes_sub seg off empty_prefixes
