(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Nativeint
open Machine
open Read_elf_common
open Util
open Readable_types
open Read_types_common


(* Machine-dependent (for IA32) *)

let name_of_section_for_types = ".types"
let name_of_section_for_codetypes = ".codetypes"

let rec read_readable_common read_one seg off =
	if off >= seg.seg_size then []
	else

	let ad = get_word seg off in
	let off = off + size_of_word in

	let len = get_word seg off in
	let len = Nativeint.to_int len in
	let off = off + size_of_word in

	let off, a = read_one seg off in
	
	let rst = read_readable_common read_one seg off in

	(ad, (a, len)) :: rst

let read_readable_heap_types =
	read_readable_common read_heap_type

let read_readable_state_types =
	read_readable_common read_state_type

let read_readable_heap_types_wrap seg = read_readable_heap_types seg 0
let read_readable_state_types_wrap seg = read_readable_state_types seg 0

let get_readable_heap_types ectx seg =
	get_sections_by_name
		name_of_section_for_types
		(Util.NativeintMap.empty : (Readable_types.heap_type * int) Util.NativeintMap.t)
		read_readable_heap_types_wrap
		Util.NativeintMap.add
		ectx
		seg

let get_readable_state_types ectx seg =
	get_sections_by_name
		name_of_section_for_codetypes
		(Util.NativeintMap.empty : (Readable_types.state_type * int) Util.NativeintMap.t)
		read_readable_state_types_wrap
		Util.NativeintMap.add
		ectx
		seg
